import { MongoStoreOptions } from './types';
import { Store, Options as RateLimitOptions, IncrementResponse, ClientRateLimitInfo } from 'express-rate-limit';
import { MongoClientOptions } from 'mongodb';
/**
 * A Store implementation to store rate limit data in MongoDB.
 */
declare class MongoStore implements Store {
    /**
     * The MongoDB client.
     */
    private client;
    /**
     * MongoDB database property.
     */
    private _db;
    /**
     * MongoDB collection property.
     */
    private _collection;
    /**
     * Window in milliseconds to limiting the requests.
     */
    windowMs: number;
    /**
     * The prefix to use for each key.
     */
    prefix: string;
    /**
     * The MongoDB URI to connect to.
     */
    uri: string;
    /**
     * The name of the collection to store rate limit data.
     */
    collectionName: string;
    /**
     * The options for the MongoDB client.
     */
    clientOptions?: MongoClientOptions | undefined;
    /**
     * If true, the store will use local keys to store rate limit data instead of the default global keys.
     */
    localKeys?: boolean | undefined;
    /**
     * Create a TTL index on the reset time field to have MongoDB automatically remove expired rate limit data.
     */
    createTtlIndex: boolean;
    /**
     * @param options The options for the store.
     */
    constructor(options: MongoStoreOptions);
    /**
     *
     * @param options The options for the Rate Limiter.
     */
    init(options: RateLimitOptions): void;
    /**
     *
     * @param uri The MongoDB URI to connect to.
     * @param options The options for the MongoDB client.
     * @returns MongoClient instance.
     */
    private createClient;
    /**
     * Prefixes the key with the store's prefix.
     * @param key The key to prefix.
     * @returns The prefixed key.
     */
    prefixKey(key: string): string;
    /**
     *
     * @param key The key to get the rate limit data for.
     * @returns ClientRateLimitInfo object or undefined if the key is not found.
     */
    get(key: string): Promise<ClientRateLimitInfo | undefined>;
    /**
     *
     * @param key The key to increment the rate limit data for.
     * @returns IncrementResponse object.
     */
    increment(key: string): Promise<IncrementResponse>;
    /**
     *
     * @param key The key to decrement the rate limit data for.
     */
    decrement(key: string): Promise<void>;
    /**
     *
     * @param key The key to reset the rate limit data for.
     */
    resetKey(key: string): Promise<void>;
    /**
     * Resets all rate limit data.
     */
    resetAll(): Promise<void>;
    /**
     * Closes the MongoDB connection.
     * @param force If true, force close the connection. Optional.
     */
    closeConnection(force?: boolean): Promise<void>;
}
export default MongoStore;
